# Copyright 2021 (c) Anna Schumaker.
import db
import sqlite3
import unittest
from gi.repository import GObject


class TestGenreTable(unittest.TestCase):
    def on_row_inserted(self, table, row):
        self.row_inserted = row

    def setUp(self):
        db.reset()

    def test_genre_table_init(self):
        self.assertIsInstance(db.genre.Table, db.genre.GenreTable)
        db.execute("SELECT genreid,plstateid,name,sort FROM genres")

    def test_genre_table_insert(self):
        db.genre.Table.connect("row-inserted", self.on_row_inserted)

        genre = db.genre.Table.insert("Test Genre")
        self.assertIsInstance(genre, db.genre.Genre)
        self.assertIsInstance(genre, db.objects.Tag)
        self.assertEqual(self.row_inserted, genre)

        self.assertEqual(genre.name, "Test Genre")
        self.assertEqual(genre.sort, "test genre")
        self.assertIsInstance(genre.playlist_state, db.state.PlaylistState)

        with self.assertRaises(sqlite3.IntegrityError):
            db.genre.Table.insert("Test Genre")

    def test_genre_table_delete(self):
        genre = db.genre.Table.find("Test Genre")
        state = genre.playlist_state
        db.genre.Table.delete(genre)
        self.assertIsNone(db.genre.Table.lookup("Test Genre"))
        self.assertIsNone(db.state.Table.get(int(state)))

    def test_genre_table_get(self):
        genre = db.genre.Table.insert("Test Genre")
        self.assertEqual(db.genre.Table.get(1), genre)
        self.assertIsNone(db.genre.Table.get(2))

    def test_genre_table_lookup(self):
        genre = db.genre.Table.insert("Test Genre")
        self.assertEqual(db.genre.Table.lookup("Test Genre"), genre)
        self.assertIsNone(db.genre.Table.lookup("none"))


class TestGenreMap(unittest.TestCase):
    def setUp(self):
        db.reset()

    def test_genre_map_init(self):
        self.assertIsInstance(db.genre.Map, db.genre.GenreMap)
        self.assertEqual(db.genre.Map.map_lhs, db.genre.Genre)
        self.assertEqual(db.genre.Map.map_rhs, db.track.Track)
        db.execute("SELECT genreid,trackid FROM genre_map")

    def test_genre_map_insert(self):
        track = db.make_fake_track(1, 1.234, "Test Title", "/a/b.cde")
        genre = db.genre.Table.find("Test Genre")

        db.genre.Map.insert(genre, track)
        with self.assertRaises(sqlite3.IntegrityError):
            db.genre.Map.insert(genre, track)

    def test_genre_map_delete(self):
        track = db.make_fake_track(1, 1.234, "Test Title", "/a/b.cde")
        genre = db.genre.Table.find("Test Genre")

        db.genre.Map.insert(genre, track)
        db.genre.Map.delete(genre, track)
        self.assertEqual(db.genre.Map.lookup_tracks(genre), [ ])

    def test_genre_map_lookup_tracks(self):
        track1 = db.make_fake_track(1, 1.234, "Test Title", "/a/b.cde")
        track2 = db.make_fake_track(2, 2.345, "Test Title 2", "/a/c.def")
        genre  = db.genre.Table.find("Test Genre")

        db.genre.Map.insert(genre, track1)
        db.genre.Map.insert(genre, track2)

        lookup_res = db.genre.Map.lookup_tracks(genre)
        self.assertEqual(lookup_res, [ track1, track2 ])

    def test_genre_map_lookup_genres(self):
        track  = db.make_fake_track(1, 1.234, "Test Title", "/a/b.cde")
        genre1 = db.genre.Table.find("Test Genre 1")
        genre2 = db.genre.Table.find("Test Genre 2")

        db.genre.Map.insert(genre1, track)
        db.genre.Map.insert(genre2, track)

        lookup_res = db.genre.Map.lookup_genres(track)
        self.assertEqual(lookup_res, [ genre1, genre2 ])

    def test_genre_map_delete_track(self):
        track  = db.make_fake_track(1, 1.234, "Test Title", "/a/b.cde")
        genre1 = db.genre.Table.find("Test Genre 1")
        genre2 = db.genre.Table.find("Test Genre 2")

        db.genre.Map.insert(genre1, track)
        db.genre.Map.insert(genre2, track)

        db.genre.Map.delete_track(track)
        self.assertEqual(db.genre.Map.lookup_genres(track), [ ])
